package com.sun.awt;

import java.awt.Component;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Shape;
import sun.awt.AWTAccessor;
import java.awt.Window;
import sun.awt.SunToolkit;
import java.awt.Toolkit;

public final class AWTUtilities
{
    private AWTUtilities() {
    }
    
    public static boolean isTranslucencySupported(AWTUtilities.Translucency translucency) {
        switch (translucency) {
            case PERPIXEL_TRANSPARENT: {
                return isWindowShapingSupported();
            }
            case TRANSLUCENT: {
                return isWindowOpacitySupported();
            }
            case PERPIXEL_TRANSLUCENT: {
                return isWindowTranslucencySupported();
            }
            default: {
                return false;
            }
        }
    }
    
    private static boolean isWindowOpacitySupported() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        return defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).isWindowOpacitySupported();
    }
    
    public static void setWindowOpacity(Window window, float n) {
        if (window == null) {
            throw new NullPointerException("The window argument should not be null.");
        }
        AWTAccessor.getWindowAccessor().setOpacity(window, n);
        return;
    }
    
    public static float getWindowOpacity(Window window) {
        if (window == null) {
            throw new NullPointerException("The window argument should not be null.");
        }
        return AWTAccessor.getWindowAccessor().getOpacity(window);
    }
    
    public static boolean isWindowShapingSupported() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        return defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).isWindowShapingSupported();
    }
    
    public static Shape getWindowShape(Window window) {
        if (window == null) {
            throw new NullPointerException("The window argument should not be null.");
        }
        return AWTAccessor.getWindowAccessor().getShape(window);
    }
    
    public static void setWindowShape(Window window, Shape shape) {
        if (window == null) {
            throw new NullPointerException("The window argument should not be null.");
        }
        AWTAccessor.getWindowAccessor().setShape(window, shape);
        return;
    }
    
    private static boolean isWindowTranslucencySupported() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if (!(defaultToolkit instanceof SunToolkit)) {
            return false;
        }
        if (!((SunToolkit)defaultToolkit).isWindowTranslucencySupported()) {
            return false;
        }
        GraphicsEnvironment localGraphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
        if (isTranslucencyCapable(localGraphicsEnvironment.getDefaultScreenDevice().getDefaultConfiguration())) {
            return true;
        }
        GraphicsDevice[] screenDevices = localGraphicsEnvironment.getScreenDevices();
        for (int i = 0; i < screenDevices.length; ++i) {
            GraphicsConfiguration[] configurations = screenDevices[i].getConfigurations();
            int j = 0;
            while (j < configurations.length) {
                if (isTranslucencyCapable(configurations[j])) {
                    return true;
                }
                ++j;
            }
        }
        return false;
    }
    
    public static void setWindowOpaque(Window window, boolean b) {
        if (window == null) {
            throw new NullPointerException("The window argument should not be null.");
        }
        if (!b && !isTranslucencySupported(AWTUtilities.Translucency.PERPIXEL_TRANSLUCENT)) {
            throw new UnsupportedOperationException("The PERPIXEL_TRANSLUCENT translucency kind is not supported");
        }
        AWTAccessor.getWindowAccessor().setOpaque(window, b);
        return;
    }
    
    public static boolean isWindowOpaque(Window window) {
        if (window == null) {
            throw new NullPointerException("The window argument should not be null.");
        }
        return window.isOpaque();
    }
    
    public static boolean isTranslucencyCapable(GraphicsConfiguration gc) {
        if (gc == null) {
            throw new NullPointerException("The gc argument should not be null");
        }
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        return defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).isTranslucencyCapable(gc);
    }
    
    public static void setComponentMixingCutoutShape(Component component, Shape shape) {
        if (component == null) {
            throw new NullPointerException("The component argument should not be null.");
        }
        AWTAccessor.getComponentAccessor().setMixingCutoutShape(component, shape);
        return;
    }
    
    public enum Translucency
    {
        PERPIXEL_TRANSPARENT, 
        TRANSLUCENT, 
        PERPIXEL_TRANSLUCENT;
    }
}
